# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2015-2020, by Garen Torikian.
# Copyright, 2016-2017, by Yuki Izumi.
# Copyright, 2017-2018, by Ashe Connor.
# Copyright, 2020-2025, by Samuel Williams.

module Markly
	VERSION = "0.15.3"
end
